;  Source name     : NEWMATH.ASM
;  Executable name : NEWMATH
;  Version         : 1.0
;  Created date    : 11/12/1999
;  Last update     : 11/27/1999
;  Author          : Jeff Duntemann
;  Description     : A simple demo of calling printf with several arguments, using NASM 0.98
;
;  Build using these commands:
;    nasm -f elf newmath.asm
;    gcc newmath.o -o newmath
;
	
		
[SECTION .text]			; Section containing code

extern printf						
global main			; Required so linker can find entry point
	
main:
        push ebp		; Set up stack frame for debugger
	mov ebp,esp
	push ebx		; Program must preserve ebp, ebx, esi, & edi
	push esi
	push edi
	;;; Everything before this is boilerplate; use it for all ordinary apps!	

	;; Here's what the first example printf call would look like coded in C:
	;;
	;; printf('%d + %d = %d...for large values of %d.',2,2,5,2);
	;;
	;; And here is how you'd pull it off in assembly for Linux:
	
	push dword 2		; Rightmost arg is pushed first
	push dword 5		; Next arg to the left
	push dword 2		; Next arg to the left
	push dword 2		; Next arg to the left
	push dword mathmsg	; Finally, push the address of the base string
	call printf		; Make the actual call to printf
	add esp,20		; Stack cleanup: 5 args x 4 bytes = 20

	;; Here's another example, this time merging strings. Coded in C:
	;;
	;; printf('Does the set of %s contain the set of %s?','mammals','dugongs');
	;;
	;; And here is how you'd pull it off in assembly for Linux:
	
	push dword dugongs	; Rightmost arg is pushed first
	push dword mammals	; Next arg to the left
	push dword setbase	; Base string is pushed last
	call printf		; Make the printf call
	add esp,12		; Stack cleanup: 3 args x 4 bytes = 12
		
	;;; Everything after this is boilerplate; use it for all ordinary apps!
	pop edi			; Restore saved registers
	pop esi
	pop ebx
	mov esp,ebp		; Destroy stack frame before returning
	pop ebp
	ret			; Return control to Linux


[SECTION .data]			; Section containing initialised data

mathmsg	db '%d + %d = %d ... for large values of %d.',10,0			
setbase db 'Does the set of %s contain the set of %s?',10,0
mammals db 'mammals',0
dugongs	db 'dugongs',0
				
[SECTION .bss]			; Section containing uninitialized data





























